#include <stdio.h>
#include "StdAfx.h"
#include "imglib.h"


BOOL ReadRAWImage(LPCTSTR pRawFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE *fp;

	if((fp = fopen(pRawFile, "rb")) == NULL) return FALSE;
	if(fread(chImage, sizeof(BYTE), (Width * Height), fp) != (unsigned)(Width * Height)){
		fclose(fp);
		return FALSE;
	}
	fclose(fp);

	return TRUE;
}

BOOL WriteRAWImage(LPCTSTR pRawFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE *fp;

	if((fp = fopen(pRawFile, "wb")) == NULL) return FALSE;
	if(fwrite(chImage, sizeof(BYTE), (Width * Height), fp) != (unsigned)(Width * Height)){
		fclose(fp);
		return FALSE;
	}
	fclose(fp);

	return TRUE;
}

BOOL ReadBMPImage(LPCTSTR pBmpFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE* fp;
	BITMAPFILEHEADER bmfHdr;
	BITMAPINFOHEADER bmiHdr;

	if((fp = fopen(pBmpFile, "rb")) == NULL) return FALSE;

	if(fread(&bmfHdr, sizeof(BYTE), sizeof(BITMAPFILEHEADER), fp) != sizeof(BITMAPFILEHEADER)) return FALSE;
	if(bmfHdr.bfType != ((WORD)'M'<<8 | 'B')) return FALSE;
	if(fread(&bmiHdr, sizeof(BYTE), sizeof(BITMAPINFOHEADER), fp) != sizeof(BITMAPINFOHEADER)) return FALSE;
	if(bmiHdr.biBitCount != 24) return FALSE;

	if(fread(chImage, sizeof(BYTE), (Width * Height * 3), fp) != (unsigned)(Width * Height * 3)) return FALSE;

	fclose(fp);

	return TRUE;
}

BOOL WriteBMPImage(LPCTSTR pBmpFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE* fp;
	BITMAPFILEHEADER bmfHdr;
	BITMAPINFOHEADER bmiHdr;
	
    MakeBMPHeader(&bmiHdr, &bmfHdr, Width, Height);

	if((fp = fopen(pBmpFile, "wb")) == NULL) return FALSE;

	if(fwrite(&bmfHdr, sizeof(BYTE), sizeof(BITMAPFILEHEADER), fp) != sizeof(BITMAPFILEHEADER)) return FALSE;
	if(fwrite(&bmiHdr, sizeof(BYTE), sizeof(BITMAPINFOHEADER), fp) != sizeof(BITMAPINFOHEADER)) return FALSE;

	if(fwrite(chImage, sizeof(BYTE), (Width * Height * 3), fp) != (unsigned)(Width * Height * 3)) return FALSE;

	fclose(fp);
	
	return TRUE;
}

void MakeBMPHeader(BITMAPINFOHEADER *pbmiHdr, BITMAPFILEHEADER *pbmfHdr, WORD Width, WORD Height)
{
	memset(pbmfHdr, 0, sizeof(BITMAPFILEHEADER));
	pbmfHdr->bfType = ((WORD)'M'<<8 | 'B');
	pbmfHdr->bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + (Width * Height * 3);
	pbmfHdr->bfReserved1 = 0;
	pbmfHdr->bfReserved2 = 0;
	pbmfHdr->bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

	memset(pbmiHdr, 0, sizeof(BITMAPINFOHEADER));
	pbmiHdr->biSize = sizeof(BITMAPINFOHEADER);
	pbmiHdr->biWidth = Width;
	pbmiHdr->biHeight = Height;
	pbmiHdr->biPlanes = 1;
	pbmiHdr->biBitCount = 24;
	pbmiHdr->biCompression = BI_RGB;
	pbmiHdr->biSizeImage = (Width * Height * 3);
}


void RAWtoBMPImage(BYTE *rawImage, BYTE *bmpImage, WORD Width, WORD Height)
{
	BYTE data;

	for(int i = 0;  i < Height; i++) {
		for(int j = 0; j < Width; j++) {
			data = rawImage[Width * i + j];

			//if (data & 0x80) data &= ~(1<<7);
			//else data |= (1<<7);

			bmpImage[((Width * (Height - (i + 1)) + j )* 3)] = data;
			bmpImage[((Width * (Height - (i + 1)) + j )* 3)+1] = data;
			bmpImage[((Width * (Height - (i + 1)) + j )* 3)+2] = data;
		}
	}
}


void RAWtoBMPImageRatio(BYTE ratio,BYTE *rawImage, BYTE *bmpImage, WORD Width, WORD Height)
{
	for(int i = 0;  i < Height; i++) {
		for(int j = 0; j < Width; j++) {
			bmpImage[((Width * (Height - (i + 1)) + j )* 3)] = ~rawImage[Width*ratio * i + j*ratio];
			bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 1] = ~rawImage[Width*ratio * i + j*ratio];
			bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 2] = ~rawImage[Width*ratio * i + j*ratio];
		}
	}
}



void BMPtoRAWImage(BYTE *bmpImage, BYTE *rawImage, WORD Width, WORD Height)
{
	for(int i = 0;  i < Height; i++) {
		for(int j = 0; j < Width; j++) {
			rawImage[Width * i + j] = (BYTE)((float)(bmpImage[((Width * (Height - (i + 1)) + j )* 3)] 
				+ bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 1]
				+ bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 2]) / 3.f);
		}
	}
}

